(function () {
  Drupal.behaviors.geofield_maplibre_gl_field_formatter = {
    attach: function (context, settings) {
      if (!(context === window.document)) {
        return;
      }
      for (const container in settings.geofieldMaplibreGlFieldFormatter ) {
        document.getElementById(container).addEventListener(
          'new-layer', (event) => {
            const map = event.detail.map;
            const mapId = event.detail.mapId;
            const layerData =  event.detail.layerData;
            if (settings.geofieldMaplibreGlFieldFormatter[container].marker_image) {
              map.loadImage(
                settings.geofieldMaplibreGlFieldFormatter[container].marker_image, function (error, image) {
                  if (error) { throw error;
                  }
                  map.addImage('marker_image', image);
                }
              );
            }
            layerData.source.data.features.forEach(
              function (point) {
                if (settings.geofieldMaplibreGlFieldFormatter[container].marker_style == "marker") {
                  var marker = new maplibregl.Marker({})
                  .setLngLat(point['geometry'].coordinates)
                  .setPopup(
                    new maplibregl.Popup({ offset: 25 })
                    .setHTML(
                      '<h2><a href="' + point.properties.geourl + '" target="_blank">Go there</a></h2>' +
                      point.properties.description
                    )
                  )
                  .addTo(map)
                }
              }
            )
            map.on(
              'click', 'points', function (e) {
                var coordinates = e.features[0].geometry.coordinates.slice();
                var description = e.features[0].properties.description +
                '<span><a href="' + e.features[0].properties.geourl + '" target="_blank">'+ Drupal.t("Open in app") + '</a></span>'
                // Ensure that if the map is zoomed out such that multiple
                // copies of the feature are visible, the popup appears
                // over the copy being pointed to.
                while (Math.abs(e.lngLat.lng - coordinates[0]) > 180) {
                  coordinates[0] += e.lngLat.lng > coordinates[0] ? 360 : -360;
                }
                new maplibregl.Popup()
                .setLngLat(coordinates)
                .setHTML(description)
                .addTo(map);
              }
            )
          }
        )
      }
    }
  };
})();
