(function (Drupal, once) {
  Drupal.behaviors.geofield_maplibre_gl_field_widget = {
    attach: function (context, settings) {
      if (!(context === window.document)) {
        return;
      }
      // the array that controls which map is already set to prevent do it
      // twice if new-layer event comes from map rather than document would help
      // but it is coming from root document. Not really used...
      maps_set = [];
      var lat_input = [];
      var lon_input = [];
      var metric_conversion_factor = [];
      var text_input = [];
      var distance_feature_collection = [];
      var on_configuration = [];

      var radius_dist = [];

      for (const picker_key in settings.geofieldMaplibreGlFieldWidget ) {
        var picker = settings.geofieldMaplibreGlFieldWidget[picker_key];
        // mandatory input form elements:
        lat_input[picker_key] = "";
        lon_input[picker_key] = "";

        radius_dist[picker_key] = "";
        //used to update paint features of the distance radius
        distance_feature_collection[picker_key] = [];

        metric_conversion_factor[picker_key] = "";
        // optional text input element to allow pick with polygon, line, etc.
        // @TODO
        text_input[picker_key] = "";

        if (once(picker_key, 'html').length) {
          document.getElementById(picker_key).addEventListener(
            'maplibre-gl.map', (event) => {
              const map = event.detail.map;
              const mapId = event.detail.mapId;
              const mapData =  event.detail.mapData;
              //{detail: {map, mapId, layerData} }
              if (/*(maps_set.indexOf(mapId) > -1) ||*/ ( mapId != picker_key ) ){
                // It will exit if map is already set or if mapid is not for this
                // picker
                return;
              }
              // disabling as it will be used to locate the marker
              map.doubleClickZoom.disable();

              // set the field identifiers to pass modifications on picker changes
              // or viceversa
              // Ideally it may come from settings, but often it must be obtained
              // via DOM selectors-
              // We need: lat lon and optionally radius_dist and text input.
              switch (settings.geofieldMaplibreGlFieldWidget[picker_key].pickerType){
                case "base_form":
                case "widget":
                  // todo.
                  // The coordinates form input elements
                  lat_input[picker_key] = context.getElementsByName(`${settings.geofieldMaplibreGlFieldWidget[picker_key].fieldName}[${settings.geofieldMaplibreGlFieldWidget[picker_key].delta}][value][lat]`)[0]
                  lon_input[picker_key] = context.getElementsByName(`${settings.geofieldMaplibreGlFieldWidget[picker_key].fieldName}[${settings.geofieldMaplibreGlFieldWidget[picker_key].delta}][value][lon]`)[0]
                  break;
                case "views_filter":
                  // set necessary context
                  let views_filter_context = map._container.closest("fieldset");

                  if (!views_filter_context) break;
                  // The radius dist element we can use
                  radius_dist[picker_key] = views_filter_context.getElementsByName(settings.geofieldMaplibreGlFieldWidget[picker_key].views_handler_id + "[value]")[0];
                metric_conversion_factor[picker_key] = settings.geofieldMaplibreGlFieldWidget[picker_key].metric_conversion_factor;
                // The coordinates form input elements
                lat_input[picker_key] = views_filter_context.getelementsbyname(settings.geofieldMaplibreGlFieldWidget[picker_key].views_handler_id + "[source_configuration][origin][lat]")[0]
                lon_input[picker_key] = views_filter_context.getelementsbyname(settings.geofieldMaplibreGlFieldWidget[picker_key].views_handler_id + "[source_configuration][origin][lon]")[0]
                // if the form is in configuration rather than exposed
                on_configuration[picker_key] = views_filter_context.getElementById("edit-instance").length;
                // The distance feature collection in its layer radius_distance
                distance_feature_collection[picker_key] = map.getSource("radius_distance").serialize().data;

                views_filter_context.getElementsByName("clear-button")[0].click( function(){
                  lat_input[picker_key].val(0);
                  lon_input[picker_key].val(0);
                  if (radius_dist[picker_key]){
                    radius_dist[picker_key].val(0);
                  }
                  lat_input[picker_key].trigger("change");
                  radius_dist[picker_key].trigger("change");
                  views_filter_context.getElementsByClass("mapboxgl-marker")[0].hide();
                });

                break;
              }

              // Pointing operations:
              if (settings.geofieldMaplibreGlFieldWidget[picker_key].marker_image){
                map.loadImage(
                  settings.geofieldMaplibreGlFieldWidget[picker_key].marker_image, function (error, image) {
                    if (error) throw error;
                    map.addImage('marker_image', image);
                  }
                );
              }


              if (settings.geofieldMaplibreGlFieldWidget[picker_key].pickerType == 'widget' || settings.geofieldMaplibreGlFieldWidget[picker_key].marker_style == "marker"){
                switch (settings.geofieldMaplibreGlFieldWidget[picker_key].pickerType){
                  case "widget":
                  case "views_filter":
                    // Add some controls
                    // will use coordinates from input instead of pointing to
                    // configurated data from
                    // point['geometry'].coordinates
                    var geolocate = new maplibregl.GeolocateControl({
                      showUserLocation: false
                    });
                    geolocate.on('geolocate', onGeolocate);
                    // @todo add geocoder as https://maplibre.org/maplibre-gl-js/docs/examples/geocoder/
                    /*map.addControl(
                     *                            new MaplibreGeocoder({
                     *                              accessToken: maplibregl.accessToken,
                     *                              maplibregl: maplibregl,
                     *                              marker: false
                })
          );*/
                    map.addControl(geolocate);

                    // Add marker
                    //TODO if circle or marker please:
                    var marker = new maplibregl.Marker({
                      draggable: true
                    })
                    .setLngLat([lon_input[picker_key].value || 0, lat_input[picker_key].value || 0])
                    .addTo(map);
                    marker.on('drag', onDrag);
                    marker.on('dragend', onDragEnd)
                    marker.on('move',onDrag)

                    map
                    .setCenter([lon_input[picker_key].value || 0, lat_input[picker_key].value || 0])
                    map.on('dblclick', onMapDblClick);

                    lat_input[picker_key].addEventListener('change', onGeofieldLanLonChange);
                    lon_input[picker_key].addEventListener('change', onGeofieldLanLonChange);


                    if (on_configuration[picker_key]){
                      //To modify behavior if we are in configuration
                    }

                    // distance radius point
                    if (distance_feature_collection[picker_key].features) {
                      distance_feature_collection[picker_key].features[0].geometry.coordinates = [lat_input[picker_key].value || 0, lon_input[picker_key].value || 0]
                      map.getSource("radius_distance").setData(distance_feature_collection[picker_key])
                    }

                    if (radius_dist[picker_key]){
                      radius_dist[picker_key].on('change', updateRadius);
                      // initialize radius on map simulating drag and change
                      radius_dist[picker_key].trigger("change")
                      marker.fire("drag")
                    }

                    function onGeofieldLanLonChange(event) {
                      marker.setLngLat([lon_input[picker_key].value || 0, lat_input[picker_key].value || 0])
                      map.setCenter([lon_input[picker_key].value || 0, lat_input[picker_key].value || 0])
                    }

                    function onMapDblClick(data){
                      if ( map._container.closest("fieldset") ) {
                        [...map._container.closest("fieldset").getElementsByClassName("maplibregl-marker")].forEach(
                          (element, index, array) => {
                            element.hidden = false
                          }
                        );
                      }
                      marker.setLngLat([data.lngLat.lng,data.lngLat.lat]);
                      // fire drag to carry on its bindings
                      marker.fire("drag");
                      marker.fire("dragend");
                    }

                    function updateRadius(data){
                      // Update the radius dist visually
                      current_latitude = marker.getLngLat().lat;
                      var radiusData = {
                        stops: [
                          [0, 0],
                          [22, metersToPixelsAtMaxZoom(metric_conversion_factor[picker_key]*this.value, current_latitude)]
                        ],
                        base: 2
                      };
                      map.setPaintProperty('radius_distance','circle-radius', radiusData);
                    }

                    function onGeolocate(position){
                      marker.setLngLat([position.coords.longitude,position.coords.latitude]);
                      // fire drag to carry on its bindings

                      marker.fire("drag");
                      marker.fire("dragend");
                    }

                    function onDrag(data){
                      // Feedback to Radius distance layer point to mirror
                      // coords
                      if (distance_feature_collection[picker_key].features){
                        distance_feature_collection[picker_key].features[0].geometry.coordinates = [data.target.getLngLat().lat, data.target.getLngLat().lng]
                        map.getSource("radius_distance").setData(distance_feature_collection[picker_key])
                      }
                    }

                    function onDragEnd() {
                      // Populate Coords input
                      var lngLat = marker.getLngLat();
                      //'Longitude: ' + lngLat.lng + '<br />Latitude: ' + lngLat.lat;
                      updateLocation(lngLat,settings.geofieldMaplibreGlFieldWidget[picker_key].pickerType);
                    }

                    function updateLocation(position, picker_type) {
                      if (!on_configuration[picker_key]){
                        const change_event = new Event('change');
                        lat_input[picker_key].value = position.lat;
                        lat_input[picker_key].dispatchEvent(change_event);
                        lon_input[picker_key].value = position.lng;
                        lon_input[picker_key].dispatchEvent(change_event);
                      }
                    }
                    break;

                }
              }
              map.on(
                'click', 'points', function (e) {
                  var coordinates = e.features[0].geometry.coordinates.slice();
                  // Ensure that if the map is zoomed out such that multiple
                  // copies of the feature are visible, the popup appears
                  // over the copy being pointed to.
                  while (Math.abs(e.lngLat.lng - coordinates[0]) > 180) {
                    coordinates[0] += e.lngLat.lng > coordinates[0] ? 360 : -360;
                  }
                  new maplibregl.Popup()
                  .setLngLat(coordinates)
                  .addTo(map);
                }
              )
              maps_set.push(mapId);
            }
          );
        }
      }
      // @todo move  On new layer to vanilla js.
      /*      $(document).on(
       *               'new-layer', function (event, map, mapData, mapId, index, layerData) {
    }
);
*/

      // @todo move On new source to vanilla js.
      /*
       *            $(document).on(
       *               'new-source', function (event, map,mapData, mapId, sourceId, sourceData) {
       *                 console.log(sourceData)
    }
);*/

      // @todo move On new map to vanilla js
      /*      $(document).on(
       *               "mapbox-gl.map", function ( event, map, mapData, mapId ) {
       *                 map.on(
       *                   'data', function (data) {
       *                     if (data.dataType === 'source' && data.isSourceLoaded) {
    }
    }
)
    }
);*/


      function metersToPixelsAtMaxZoom(meters, latitude){
        // ~0.075m/px (meters per pixel)
        return (meters / 0.075 / Math.cos(latitude * Math.PI / 180)) * 4
      }
      function convertUnits(){
      }
    }
  };
})(Drupal, once);
